@@ -26,7 +26,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# along with this program. If not, see <https://www.gnu.org/licenses/>.
 
 use 5.008;  # We're using PerlIO layers
 use strict;
@@ -35,13 +35,14 @@ use open ':utf8';  # changelogs are written with UTF-8 encoding
 use filetest 'access';  # use access rather than stat for -w
 # for checking whether user names are valid and making format() behave
 use Encode qw/decode_utf8 encode_utf8/;
-use Getopt::Long qw(:config gnu_getopt);
+use Getopt::Long qw(:config bundling permute no_getopt_compat);
 use File::Copy;
 use File::Basename;
 use Cwd;
 use Dpkg::Vendor qw(get_current_vendor);
-use Dpkg::Changelog::Parse;
-BEGIN { push @INC, '/usr/share/devscripts'; }
+use Dpkg::Changelog::Parse qw(changelog_parse);
+use Dpkg::Control;
+use Dpkg::Version;
 use Devscripts::Compression;
 use Devscripts::Debbugs;
 use POSIX qw(locale_h strftime);
@@ -60,25 +61,6 @@ my $CHGLINE;  # used by the format O section at the end
 
 my $compression_re = compression_get_file_extension_regex();
 
-my $lpdc_broken;
-
-sub have_lpdc {
-    return ($lpdc_broken ? 0 : 1) if defined $lpdc_broken;
-    eval {
-	require Parse::DebControl;
-    };
-
-    if ($@) {
-	if ($@ =~ m%^Can\'t locate Parse/DebControl%) {
-	    $lpdc_broken="the libparse-debcontrol-perl package is not installed";
-	} else {
-	    $lpdc_broken="couldn't load Parse::DebControl: $@";
-	}
-    }
-    else { $lpdc_broken=''; }
-    return $lpdc_broken ? 0 : 1;
-}
-
 my $debian_distro_info;
 sub get_debian_distro_info {
     return $debian_distro_info if defined $debian_distro_info;
@@ -179,7 +161,7 @@ Options:
          distribution name
   --bpo
          Increment the Debian release number for a backports upload
-         to "wheezy-backports"
+         to "stretch-backports"
   -l, --local <suffix>
          Add a suffix to the Debian version number for a local build
   -b, --force-bad-version
@@ -490,7 +472,7 @@ if ($vendor eq 'Ubuntu' and ($opt_n or $opt_bn or $opt_qa or $opt_bpo)) {
 # Check the distro name given.
 if (defined $opt_D) {
     if ($vendor eq 'Debian') {
-	unless ($opt_D =~ /^(experimental|unstable|UNRELEASED|((old)?stable|testing)(-proposed-updates|-security)?|proposed-updates)$/) {
+	unless ($opt_D =~ /^(experimental|unstable|sid|UNRELEASED|((old){0,2}stable|testing|wheezy|jessie|stretch|buster|bullseye)(-proposed-updates|-security)?|proposed-updates)$/) {
 	    my $deb_info = get_debian_distro_info();
 	    my ($oldstable_backports, $stable_backports) = ("", "");
 	    if ($deb_info == 0) {
@@ -505,9 +487,12 @@ if (defined $opt_D) {
 	    if ($deb_info == 0 || $opt_D !~ m/^(\Q$stable_backports\E|\Q$oldstable_backports\E)$/) {
 		$stable_backports = ", " . $stable_backports if $stable_backports;
 		$oldstable_backports = ", " . $oldstable_backports if $oldstable_backports;
-		warn "$progname warning: Recognised distributions are: unstable, testing, stable,\n"
-		     . "oldstable, experimental, {testing-,stable-,oldstable-,}proposed-updates,\n"
-		     . "{testing,stable,oldstable}-security$oldstable_backports$stable_backports and UNRELEASED.\n"
+		warn "$progname warning: Recognised distributions are: \n"
+		     . "experimental, unstable, testing, stable, oldstable, oldoldstable,\n"
+                     . "{bullseye,buster,stretch,jessie,wheezy}-proposed-updates,\n"
+		     . "{testing,stable,oldstable,oldoldstable}-proposed-updates,\n"
+                     . "{bullseye,buster,stretch,jessie,wheezy}-security,\n"
+		     . "{testing,stable,oldstable,oldoldstable}}-security$oldstable_backports$stable_backports and UNRELEASED.\n"
 		     . "Using your request anyway.\n";
 		$warnings++ if not $opt_force_dist;
 	    }
@@ -624,15 +609,15 @@ my $MAINTAINER = 'MAINTAINER';
 my $EMAIL = 'EMAIL';
 my $DISTRIBUTION = 'UNRELEASED';
 my $bpo_dist = '';
-my %bpo_dists = ( 60, 'squeeze', 70, 'wheezy' );
-my $latest_bpo_dist = '70';
+my %bpo_dists = ( 60, 'squeeze', 70, 'wheezy', 8, 'jessie', 9, 'stretch' );
+my $latest_bpo_dist = '9';
 my $CHANGES = '';
 # Changelog urgency, possibly propogated to NEWS files
 my $CL_URGENCY = '';
 
 if (! $opt_create || ($opt_create && $opt_news)) {
     my $file = $opt_create ? 'debian/changelog' : $changelog_path;
-    $changelog = Dpkg::Changelog::Parse::changelog_parse(file => $file);
+    $changelog = changelog_parse(file => $file);
 
     # Now we've read the changelog, set some variables and then
     # let's check the directory name is sensible
@@ -645,7 +630,8 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     fatal "No maintainer in changelog!"
 	unless exists $changelog->{Maintainer};
     $changelog->{Maintainer} = decode_utf8($changelog->{Maintainer});
-    ($MAINTAINER,$EMAIL) = ($changelog->{Maintainer} =~ /^([^<]+) <(.*)>/);
+    ($MAINTAINER,$EMAIL) = ($changelog->{Maintainer} =~ /^([^<]*) <(.*)>/);
+    $MAINTAINER ||= '';
     fatal "No distribution in changelog!"
 	unless exists $changelog->{Distribution};
     if ($vendor eq 'Ubuntu') {
@@ -662,7 +648,7 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     if ($opt_news) {
 	my $found_version = 0;
 	my $found_urgency = 0;
-	my $clog = Dpkg::Changelog::Parse::changelog_parse(file => $real_changelog_path);
+	my $clog = changelog_parse(file => $real_changelog_path);
 	$VERSION = $clog->{Version};
 	$VERSION =~ s/~$//;
 
@@ -813,18 +799,14 @@ if (! $opt_m and ! $opt_M) {
 
 if ($opt_M) {
     if (-f 'debian/control') {
-	if (have_lpdc()) {
-	    my $parser = Parse::DebControl->new;
-	    my $deb822 = $parser->parse_file('debian/control', {stripComments => 'true'});
-	    my $maintainer = decode_utf8($deb822->[0]->{'Maintainer'});
-	    if ($maintainer =~ /^(.*)\s+<(.*)>$/) {
-		$MAINTAINER = $1;
-		$EMAIL = $2;
-	    } else {
-		fatal "$progname: invalid debian/control Maintainer field value\n";
-	    }
+	my $parser = Dpkg::Control->new(type => CTRL_INFO_SRC);
+	$parser->load('debian/control');
+	my $maintainer = decode_utf8($parser->{Maintainer});
+	if ($maintainer =~ /^(.*)\s+<(.*)>$/) {
+	    $MAINTAINER = $1;
+	    $EMAIL = $2;
 	} else {
-	    fatal "$progname: unable to get maintainer from debian/control: $lpdc_broken\n";
+	    fatal "$progname: invalid debian/control Maintainer field value\n";
 	}
     } else {
 	fatal "Missing file debian/control";
@@ -843,23 +825,19 @@ if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
        $changelog->{Distribution} eq 'UNRELEASED' and ! $opt_i_passed)) {
 
     if (-f 'debian/control') {
-	if (have_lpdc()) {
-	    my $parser = new Parse::DebControl;
-	    my $deb822 = $parser->parse_file('debian/control', {stripComments => 'true'});
-	    my $uploader = decode_utf8($deb822->[0]->{'Uploaders'}) || '';
-	    my $maintainer = decode_utf8($deb822->[0]->{'Maintainer'});
-	    my @uploaders = split(/,\s*/, $uploader);
+	my $parser = Dpkg::Control->new(type => CTRL_INFO_SRC);
+	$parser->load('debian/control');
+	my $uploader = decode_utf8($parser->{Uploaders}) || '';
+	my $maintainer = decode_utf8($parser->{Maintainer});
+	my @uploaders = split(/,\s*/, $uploader);
 
-	    my $packager = "$MAINTAINER <$EMAIL>";
+	my $packager = "$MAINTAINER <$EMAIL>";
 
-	    if ($maintainer !~ m/<packages\@qa\.debian\.org>/ and
-		! grep { $_ eq $packager } ($maintainer, @uploaders) and
-		$packager ne $changelog->{Maintainer} and ! $opt_team) {
-		$opt_n=1;
-		$opt_a=0;
-	    }
-	} else {
-	    warn "$progname: skipping automatic NMU detection: $lpdc_broken\n";
+	if ($maintainer !~ m/<packages\@qa\.debian\.org>/ and
+	    ! grep { $_ eq $packager } ($maintainer, @uploaders) and
+	    $packager ne $changelog->{Maintainer} and ! $opt_team) {
+	    $opt_n=1;
+	    $opt_a=0;
 	}
     } else {
 	fatal "Missing file debian/control";
@@ -1096,8 +1074,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 	    $opt_p=1;
 	}
 
-	if (system("dpkg --compare-versions $VERSION le $NEW_VERSION" .
-		  " 2>/dev/null 1>&2")) {
+	if (version_compare($VERSION, $NEW_VERSION) == 1) {
 	    if ($opt_b or ($opt_allow_lower and $NEW_VERSION =~ /$opt_allow_lower/)) {
 		warn "$progname warning: new version ($NEW_VERSION) is less than\n" .
 		    "the current version number ($VERSION).\n";
@@ -1244,8 +1221,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 
     if (($opt_v or $opt_i or $opt_l or $opt_d) and
 	$opt_release_heuristic eq 'changelog' and
-	$changelog->{Distribution} eq 'UNRELEASED' and
-	$distribution eq 'UNRELEASED') {
+	$changelog->{Distribution} eq 'UNRELEASED') {
 
 	$merge = 1;
     } else {
@@ -1277,7 +1253,7 @@ if (($opt_i || $opt_n || $opt_bn || $opt_qa || $opt_R || $opt_s || $opt_team ||
 	    print O "  * Rebuild for $bpo_dist.\n";
 	    $line = 1;
 	}
-	if (@closes_text or $TEXT) {
+	if (@closes_text or $TEXT or $EMPTY_TEXT) {
 	    foreach (@closes_text) { format_line($_, 1); }
 	    if (length $TEXT) { format_line($TEXT, 1); }
 	} elsif ($opt_news) {
@@ -1342,7 +1318,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 	    ($dist_indicator = $1) =~ s/[!:.,;]$//;
 	    chomp $dist_indicator;
 	}
-	elsif (/^ --\s+([^<]+)\s+/) {
+	elsif (/^ --\s+([^<]+)\s+/ || /^ --\s+<(.+?)>/) {
 	    $lastmaint=$1;
 	    # Remember where we are so we can skip back afterwards
 	    $savedline = $line;
@@ -1580,24 +1556,18 @@ copy("$changelog_path.dch","$changelog_path") or
 if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
     !$opt_p && !$opt_create) {
     # Find the current version number etc.
-    my ($new_version, $new_sversion, $new_uversion);
-    open PARSED, "dpkg-parsechangelog |"
-	or fatal "Cannot execute dpkg-parsechangelog: $!";
-    while (<PARSED>) {
-	if (/^Version:\s(.+?)\s*$/) { $new_version=$1; }
+    my $v;
+    my $changelog = changelog_parse();
+    if (exists $changelog->{Version}) {
+	$v = Dpkg::Version->new($changelog->{Version});
     }
 
-    close PARSED
-	or fatal "Problem executing dpkg-parsechangelog: $!";
-    if ($?) { fatal "dpkg-parsechangelog failed!" }
-
     fatal "No version number in debian/changelog!"
-	unless defined $new_version;
+	unless $v;
 
-    # Is this a native Debian package, i.e., does it have a - in the
-    # version number?
-    $new_version =~ s/^\d+://;  # remove epoch
-    ($new_uversion=$new_version) =~ s/-[^-]*$//;  # remove revision
+    my ($new_version, $new_uversion);
+    $new_version = $v->as_string(omit_epoch => 1);
+    $new_uversion = $v->as_string(omit_epoch => 1, omit_revision => 1);
 
     if ($new_uversion ne $UVERSION) {
 	# Then we rename the directory
@@ -1606,11 +1576,13 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	} else {
 	    warn "$progname warning: Couldn't rename directory: $!\n";
 	}
-	# And check whether a new orig tarball exists
-	my @origs = glob("../$PACKAGE\_$new_uversion.*");
-	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re$/ } @origs;
-	if ($num_origs == 0) {
-	    warn "$progname warning: no orig tarball found for the new version.\n";
+	if (!$v->is_native()) {
+	    # And check whether a new orig tarball exists
+	    my @origs = glob("../$PACKAGE\_$new_uversion.*");
+	    my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re$/ } @origs;
+	    if ($num_origs == 0) {
+		warn "$progname warning: no orig tarball found for the new version.\n";
+	    }
 	}
     }
 }
@@ -1642,7 +1614,7 @@ sub format_line {
 
     # Work around the fact that write() with formats
     # seems to assume that characters are single-byte
-    # See http://rt.perl.org/rt3/Public/Bug/Display.html?id=33832
+    # See https://rt.perl.org/Public/Bug/Display.html?id=33832
     # and Debian bugs #473769 and #541484
     # This relies on $CHGLINE being a sequence of unicode characters.  We can
     # compare how many unicode characters we have to how many bytes we have
