@@ -17,14 +17,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# along with this program. If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
-BEGIN { push @INC, '/usr/share/devscripts'; }
 use Devscripts::Packages;
 use File::Basename;
-use Getopt::Long qw(:config gnu_getopt);
+use File::Copy qw(move);
+use File::HomeDir;
+use File::Path qw(make_path);
+use File::Spec;
+use Getopt::Long qw(:config bundling permute no_getopt_compat);
 
 sub remove_duplicate_values($);
 sub store_if_relevant(%);
@@ -32,9 +35,12 @@ sub human_flags($);
 sub unhtmlsanit($);
 sub dt_parse_request($);
 
-my $cachedir = $ENV{'HOME'}."/.devscripts_cache/";
+$ENV{HOME} = File::HomeDir->my_home;
+my $cachedir = $ENV{XDG_CACHE_HOME} || File::Spec->catdir($ENV{HOME}, '.cache');
+$cachedir = File::Spec->catdir($cachedir, 'devscripts', 'rc-alert');
+
 my $url = "http://bugs.debian.org/release-critical/other/all.html";
-my $cachefile = $cachedir . basename($url);
+my $cachefile = File::Spec->catfile($cachedir, basename($url));
 my $forcecache = 0;
 my $usecache = 0;
 
@@ -172,10 +178,15 @@ if (system("command -v wget >/dev/null 2>&1") == 0) {
     die "$progname: this program requires either the wget or curl package to be installed\n";
 }
 
-
-if (! -d $cachedir and $forcecache) {
-    mkdir $cachedir
-	or die "$progname: can't make cache directory $cachedir: $!\n";
+# TODO: Remove oldcache handling post-Stretch
+my $oldcache = File::Spec->catfile($ENV{HOME}, '.devscripts_cache', 'all.html');
+if (! -d $cachedir) {
+    if (-f $oldcache || $forcecache) {
+	make_path($cachedir);
+	if (-f $oldcache) {
+	    move($oldcache, $cachefile);
+	}
+    }
 }
 
 if (-d $cachedir) {
@@ -325,6 +336,7 @@ sub store_if_relevant(%) {
 	my $bug_string = "Package: $pkgname\n" .
 	    $comment .  # non-empty comments always contain the trailing \n
 	    "Bug:     $args{num}\n" .
+	    "Bug-URL: https://bugs.debian.org/$args{num}\n" .
 	    "Title:   " . unhtmlsanit($args{name}) . "\n" .
 	    "Flags:   " . $flags . "\n" .
 	    (defined $args{dists} ? "Dists:  " . $dists . "\n" : "") .
@@ -423,7 +435,7 @@ sub human_dists($) {
     }
 }
 
-# Reverse of master.debian.org:/org/bugs.debian.org/cgi-bin/common.pl
+# Reverse of master.debian.org:/srv/bugs.debian.org/cgi-bin/common.pl
 sub unhtmlsanit ($) {
     my %saniarray = ('lt','<', 'gt','>', 'amp','&', 'quot', '"');
     my $in = $_[0];
