@@ -1,7 +1,6 @@
 #!/usr/bin/python3
 
 from setuptools import setup
-import glob
 import os
 import re
 
@@ -13,16 +12,13 @@ if os.path.exists(changelog):
     if match:
         version = match.group(1)
 
-scripts = ['suspicious-source',
-           'wrap-and-sort',
-          ]
+scripts = "debdiff-apply sadt suspicious-source wrap-and-sort".split()
 
 if __name__ == '__main__':
-    setup(name='devscripts',
-          version=version,
-          scripts=scripts,
-          packages=['devscripts',
-                    'devscripts/test',
-                   ],
-          test_suite='devscripts.test.discover',
+    setup(
+        name='devscripts',
+        version=version,
+        scripts=scripts,
+        packages=['devscripts', 'devscripts/test'],
+        test_suite='devscripts.test.discover',
     )
